@@ -8,6 +8,7 @@ use PAR::Filter;
 use Filter::Crypto::CryptFile;
 use Pcore::Src::File;
 use Config;
+use Fcntl qw[:DEFAULT SEEK_END];
 
 has dist   => ( is => 'ro', isa => InstanceOf ['Pcore::Dist'],       required => 1 );
 has script => ( is => 'ro', isa => InstanceOf ['Pcore::Util::Path'], required => 1 );
@@ -55,7 +56,7 @@ sub _build_exe_filename ($self) {
 }
 
 sub run ($self) {
-    say qq[\nBuilding ] . ( $self->crypt ? BLACK ON_GREEN . ' crypted ' : BOLD WHITE ON_RED . q[ not crypted ] ) . RESET . qq[ "@{[$self->exe_filename]}" for $Config{archname}$LF];
+    say qq[\nBuilding ] . ( $self->crypt ? BLACK ON_GREEN . ' crypted ' : BOLD WHITE ON_RED . q[ not crypted ] ) . RESET . q[ ] . BLACK ON_GREEN . ( $self->clean ? ' clean ' : ' not clean ' ) . RESET . qq[ "@{[$self->exe_filename]}" for $Config{archname}$LF];
 
     # add main script
     $self->_add_perl_source( $self->script->realpath->to_string, 'script/main.pl' );
@@ -121,7 +122,7 @@ sub run ($self) {
     # create parl executable
     my $parl_path = P->file->temppath( suffix => $self->par_suffix );
 
-    print 'write parl ... ';
+    print 'writing parl ... ';
 
     my $cmd = qq[parl -B -O$parl_path ] . $zip_fh->path;
 
@@ -129,13 +130,13 @@ sub run ($self) {
 
     say 'done';
 
-    my $repacked_fh = $self->_repack_parl( $parl_path, $zip );
+    my $repacked_path = $self->_repack_parl( $parl_path, $zip );
 
     my $target_exe = $self->dist->root . 'data/' . $self->exe_filename;
 
-    P->file->move( $repacked_fh->path, $target_exe );
+    P->file->move( $repacked_path, $target_exe );
 
-    P->file->chmod( 'r-x------', $target_exe );
+    P->file->chmod( 'rwx------', $target_exe );
 
     say 'final binary size: ' . BLACK ON_GREEN . q[ ] . format_num( -s $target_exe ) . q[ ] . RESET . ' bytes';
 
@@ -194,6 +195,24 @@ sub _add_share ($self) {
 
 sub _add_modules ($self) {
 
+    # add full unicore database
+    for my $lib ( reverse @INC ) {
+        if ( -d "$lib/unicore/" ) {
+            P->file->find(
+                "$lib/unicore/",
+                abs => 1,
+                dir => 0,
+                sub ($path) {
+                    return if $path !~ /[.]p[lm]\z/sm;
+
+                    $self->_add_module($path);
+
+                    return;
+                }
+            );
+        }
+    }
+
     # add .pl, .pm
     for my $module ( grep {/[.](?:pl|pm)\z/sm} keys $self->mod->%* ) {
         my $found = $self->_add_module($module);
@@ -426,35 +445,29 @@ sub _compress_upx ( $self, $path ) {
 }
 
 sub _repack_parl ( $self, $parl_path, $zip ) {
-    say 'repack parl ... ';
+    print 'repacking parl ... ';
 
     my $src = P->file->read_bin($parl_path);
 
     my $in_len = length $src->$*;
 
-    my $hash = P->digest->md5_hex( $src->$* );
-
-    # find zip length
+    # cut magic string
     $src->$* =~ s/(.{4})\x0APAR[.]pm\x0A\z//sm;
 
-    my $zip_overlay_length = unpack 'N', $1;
+    # unpack overlay length
+    my $overlay_length = unpack 'N', $1;
 
-    # cut zip overlay
-    my $overlay = substr $src->$*, length( $src->$* ) - $zip_overlay_length, $zip_overlay_length, q[];
+    # extract overlay
+    # src = raw exe header
+    # overlay = files sections + par zip section + cache id string
+    my $overlay = substr $src->$*, length( $src->$* ) - $overlay_length, $overlay_length, q[];
 
-    # cut CACHE_ID, now $overlay contains only parl embedded files
+    # cut cache id, now overlay = files sections + par zip section
     $overlay =~ s/.{40}\x{00}CACHE\z//sm;
 
-    # repacked_exe_fh contains raw exe header, without overlay
-    my $repacked_exe_fh = P->file->tempfile( suffix => $self->par_suffix );
-
-    my $exe_header_length = length $src->$*;
-
-    P->file->write_bin( $repacked_exe_fh, $src );
-
     my $parl_so_temp = P->file->tempdir;
 
-    my $parl_so_temp_map = {};
+    my $file_section = {};
 
     while (1) {
         last if $overlay !~ s/\AFILE//sm;
@@ -472,16 +485,17 @@ sub _repack_parl ( $self, $parl_path, $zip ) {
         if ( $filename =~ /[.](?:pl|pm)\z/sm ) {
 
             # compress perl sources
-            $content = Pcore::Src::File->new(
+            $file_section->{$filename} = Pcore::Src::File->new(
                 {   action      => 'compress',
                     path        => $filename,
                     is_realpath => 0,
                     in_buffer   => \$content,
                     filter_args => {             #
-                        perl_compress => 1,
+                        perl_compress         => 1,
+                        perl_compress_keep_ln => 0,
                     },
                 }
-            )->run->out_buffer->$*;
+            )->run->out_buffer;
         }
         elsif ( $self->upx && $filename =~ /[.]$Config{so}\z/sm ) {
 
@@ -490,70 +504,91 @@ sub _repack_parl ( $self, $parl_path, $zip ) {
 
             P->file->write_bin( $temppath, $content );
 
-            # save mapping for temppath -> parl filename
-            $parl_so_temp_map->{$temppath} = $filename;
-
-            # will be compressed and added later
-            next;
+            $file_section->{$filename} = $temppath->path;
+        }
+        else {
+            $file_section->{$filename} = \$content;
         }
-
-        # pack file back to the overlay
-        print {$repacked_exe_fh} 'FILE' . pack( 'N', length($filename) + 9 ) . sprintf( '%08x', Archive::Zip::computeCRC32($content) ) . q[/] . $filename . pack( 'N', length $content ) . $content;
     }
 
+    my $path = P->file->temppath( suffix => $self->par_suffix );
+
+    # write raw exe
+    P->file->write_bin( $path, $src );
+
+    # patch windows exe icon, need to patch before upx
+    $self->_patch_icon($path);
+
+    my $md5 = Digest::MD5->new;
+
     if ( $self->upx ) {
-        $self->_compress_upx( [ keys $parl_so_temp_map->%* ] );
+        $self->_compress_upx( [ $path, grep { !ref } values $file_section->%* ] );
 
-        P->file->find(
-            $parl_so_temp,
-            abs => 1,
-            dir => 0,
-            sub ($path) {
-                my $content = P->file->read_bin($path)->$*;
-
-                my $filename = $parl_so_temp_map->{$path};
-
-                print {$repacked_exe_fh} 'FILE' . pack( 'N', length($filename) + 9 ) . sprintf( '%08x', Archive::Zip::computeCRC32($content) ) . q[/] . $filename . pack( 'N', length $content ) . $content;
-
-                return;
-            }
-        );
+        $md5->add( P->file->read_bin($path)->$* );
+    }
+    else {
+        $md5->add( $src->$* );
     }
 
-    # add par itself
-    $zip->writeToFileHandle($repacked_exe_fh);
+    my $fh = P->file->get_fh( $path, O_RDWR );
 
-    # write magic strings
-    print {$repacked_exe_fh} pack( 'Z40', $hash ) . qq[\x00CACHE];
+    $fh->seek( 0, SEEK_END );
 
-    print {$repacked_exe_fh} pack( 'N', $repacked_exe_fh->tell - $exe_header_length ) . "\x0APAR.pm\x0A";
+    my $exe_header_length = $fh->tell;
 
-    my $out_len = $repacked_exe_fh->tell;
+    # adding files sections
+    for my $filename ( sort keys $file_section->%* ) {
+        my $content = ref $file_section->{$filename} ? $file_section->{$filename} : P->file->read_bin( $file_section->{$filename} );
 
-    say 'parl repacked: ', BLACK ON_GREEN . q[ ] . format_num( $out_len - $in_len ) . q[ ] . RESET . ' bytes';
+        $fh->print( 'FILE' . pack( 'N', length($filename) + 9 ) . sprintf( '%08x', Archive::Zip::computeCRC32( $content->$* ) ) . q[/] . $filename . pack( 'N', length $content->$* ) . $content->$* );
+
+        $md5->add( $content->$* );
+    }
+
+    # addding par zip section, handle should be opened in r/w mode
+    $zip->writeToFileHandle( $fh, 1 ) and die;
+
+    # calculate par zip section hash
+    for my $member ( sort { $a->fileName cmp $b->fileName } $zip->members ) {
+        $md5->add( $member->fileName . $member->crc32String );
+    }
+
+    my $hash = $md5->hexdigest;
+
+    # writing cache id
+    $fh->print( pack( 'Z40', $hash ) . qq[\x00CACHE] );
+
+    # writing overlay length
+    $fh->print( pack( 'N', $fh->tell - $exe_header_length ) . "\x0APAR.pm\x0A" );
+
+    my $out_len = $fh->tell;
+
+    say 'done, ', BLACK ON_GREEN . q[ ] . format_num( $out_len - $in_len ) . q[ ] . RESET . ' bytes';
+
+    say 'hash: ' . $hash;
 
     # need to close fh before copy / patch file
-    $repacked_exe_fh->close;
+    $fh->close;
+
+    return $path;
+}
+
+sub _patch_icon ( $self, $path ) {
+
+    # .ico
+    # 4 layers, 16x16, 32x32, 16x16, 32x32
+    # all layers 8bpp, 1-bit alpha, 256-slot palette
 
-    # patch windows exe icon
     if ($MSWIN) {
-
-        # .ico
-        # 4 layers, 16x16, 32x32, 16x16, 32x32
-        # all layers 8bpp, 1-bit alpha, 256-slot palette
-
-        print 'patch win exe icon ... ';
-
         state $init = !!require Win32::Exe;
 
-        my $exe = Win32::Exe->new( $repacked_exe_fh->path );
+        # path should be passed as plain string
+        my $exe = Win32::Exe->new("$path");
 
         $exe->update( icon => $ENV->share->get('/data/par.ico') );
-
-        say 'done';
     }
 
-    return $repacked_exe_fh;
+    return;
 }
 
 sub _error ( $self, $msg ) {
@@ -569,20 +604,18 @@ sub _error ( $self, $msg ) {
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ## | Sev. | Lines                | Policy                                                                                                         |
 ## |======+======================+================================================================================================================|
-## |    3 | 179, 198, 205, 237,  | References::ProhibitDoubleSigils - Double-sigil dereference                                                    |
-## |      | 312, 353, 505        |                                                                                                                |
+## |    3 | 180, 217, 224, 256,  | References::ProhibitDoubleSigils - Double-sigil dereference                                                    |
+## |      | 331, 372, 525, 540   |                                                                                                                |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 251                  | Subroutines::ProhibitManyArgs - Too many arguments                                                             |
+## |    3 | 270                  | Subroutines::ProhibitManyArgs - Too many arguments                                                             |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 387, 405             | ValuesAndExpressions::ProhibitMismatchedOperators - Mismatched operator                                        |
+## |    3 | 406, 424             | ValuesAndExpressions::ProhibitMismatchedOperators - Mismatched operator                                        |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    3 | 440                  | RegularExpressions::ProhibitCaptureWithoutTest - Capture variable used outside conditional                     |
+## |    3 | 458                  | RegularExpressions::ProhibitCaptureWithoutTest - Capture variable used outside conditional                     |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    2 | 475                  | ValuesAndExpressions::ProhibitLongChainsOfMethodCalls - Found method-call chain of length 4                    |
+## |    2 | 559, 562             | ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       |
 ## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    2 | 527, 529             | ValuesAndExpressions::ProhibitEscapedCharacters - Numeric escapes in interpolated string                       |
-## |------+----------------------+----------------------------------------------------------------------------------------------------------------|
-## |    1 | 462, 468             | CodeLayout::ProhibitParensWithBuiltins - Builtin function called with parentheses                              |
+## |    1 | 475, 481             | CodeLayout::ProhibitParensWithBuiltins - Builtin function called with parentheses                              |
 ## +------+----------------------+----------------------------------------------------------------------------------------------------------------+
 ##
 ## -----SOURCE FILTER LOG END-----
